@@ -1,20 +1,21 @@
-
-/* Copyright (C) 2009  Roberto Bertolusso, Marek Kimmel
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, a copy is available at
-   http://www.r-project.org/Licenses/
-*/
+// -*- mode: C; c-indent-level: 2; c-basic-offset: 2; tab-width: 8 -*-
+//
+// Copyright (C) 2009-2014 Roberto Bertolusso and Marek Kimmel
+//
+// This file is part of bioPN.
+//
+// bioPN is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// bioPN is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with bioPN. If not, see <http://www.gnu.org/licenses/>.
 
 #include <R.h>
 #include <Rinternals.h>
@@ -23,12 +24,14 @@
 #include <R_ext/Visibility.h>
 
 #include "quicksort.h"
+#include "helper.h"
 
 #define PRODUCTS_MAXIMUM_AMOUNT 100000000
 #define HAZARD_VALUE_TO_IGNORE 1e-10
 #define ZERO 1e-60
 
 #define RB_TIME
+#define RB_SUBTIME
 
 #ifdef RB_TIME
 #include <time.h>
@@ -242,7 +245,11 @@ SEXP GibsonBruck(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
   SEXP sexpD_time;
   PROTECT(sexpD_time = allocVector(REALSXP, iTotalSteps));
   double *pdDiscTime = REAL(sexpD_time);
-  pdDiscTime[0] = 0;
+  double dTmp = 0;
+  for (k = 0; k < iTotalSteps; k++) {
+    pdDiscTime[k] = dTmp;
+    dTmp += dDelta;
+  }
 
   SEXP sexpMarkingRowNames;
   PROTECT(sexpMarkingRowNames = allocVector(INTSXP, iTotalSteps));
@@ -332,15 +339,9 @@ SEXP GibsonBruck(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
     int iTotTransitions = 0;
 
     int iStep = 0;
+    int iInterruptCnt = 10000000;
     double dNewHazard = 0;
     do {
-      if (iStep) {
-	--iStep;
-	for(iPlace = 0; iPlace < iPlaces; iPlace++) {
-	  pdCrntMarking[iPlace] = ppdMarking[iPlace][iStep];
-	}
-      }
-      
       if (pCManage_time || sexpRManage_time) {
 	double dEnd = 0;
 	if (pCManage_time) {
@@ -360,11 +361,18 @@ SEXP GibsonBruck(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 	}
 	iSectionSteps = (int)(dEnd / dDelta) + 1;
       }
-      dTime = dTarget;
 
+      for(iPlace = 0; iPlace < iPlaces; iPlace++) {
+	ppdMarking[iPlace][iStep] = pdCrntMarking[iPlace];
+      }
+
+      dTime = dTarget;
+      dTarget += dDelta;
+      
       // For the calculation of all hazards...
       int iLastTransition = iTransitions;
-      do {    
+      
+      do {
 	// Get hazards only for the transitions associated with
 	// places whose quantities changed in the last step.
 	for(iTransitionPtr = 0; iTransitionPtr < piHazardsToModxRowTot[iLastTransition]; iTransitionPtr++) {
@@ -378,8 +386,8 @@ SEXP GibsonBruck(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 		dNewHazard *= (pdCrntMarking[iPlace] - k) / (double)(k+1);
 	    }
 	    break;
-	  case HZ_CFUNCTION:	
-	    dNewHazard = ((double(*)(double *)) pCFunction[iTransition])(pdCrntMarking);
+	  case HZ_CFUNCTION:
+	    dNewHazard = ((double(*)(double, double *)) pCFunction[iTransition])(dTime, pdCrntMarking);
 	    break;
 	  case HZ_RFUNCTION:
 	    defineVar(install("y"), sexpCrntMarking, rho);
@@ -450,16 +458,22 @@ SEXP GibsonBruck(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 	dTime = pdNextTime[iLastTransition = pnodeRoot->iTransition];
 	
 	while (dTime >= dTarget) {
-	  if (!iRun)
-	    pdDiscTime[iStep] = dTarget;
+	  ++iStep;
 	  // Update the state for the fixed incremented time.
 	  for(iPlace = 0; iPlace < iPlaces; iPlace++)
 	    ppdMarking[iPlace][iStep] = pdCrntMarking[iPlace];
-	  if (++iStep >= iSectionSteps)
+	  if (iStep == iSectionSteps - 1)
 	    goto EXIT_LOOP;
+
 	  dTarget += dDelta;
+
+	  // Force check if user interrupted
+	  iInterruptCnt = 1;
+	}
+	if (! --iInterruptCnt) {
 	  // Allow user interruption
 	  R_CheckUserInterrupt();
+	  iInterruptCnt = 10000000;
 	}
 	//piTotTransitions[iLastTransition]++;
 	for(iPlacePtr = 0; iPlacePtr < piSNZxRowTot[iLastTransition]; iPlacePtr++) {
@@ -468,12 +482,19 @@ SEXP GibsonBruck(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 	  // Update the state
 	  pdCrntMarking[iPlace] += piS[iLastTransition + iTransitions * iPlace];
 	}
-      } while (++iTotTransitions);
+	++iTotTransitions;
+      } while (TRUE);
     EXIT_LOOP:;
       Rprintf(".");
     } while (iSectionSteps < iTotalSteps);
     iTotalUsedRandomNumbers += iUsedRandomNumbers;
-    Rprintf("\t%d\t%d\t%d\n", iTotTransitions, iUsedRandomNumbers, iTotalUsedRandomNumbers);
+    Rprintf("\t%d\t%d\t%d", iTotTransitions, iUsedRandomNumbers, iTotalUsedRandomNumbers);
+#ifdef RB_SUBTIME
+    c1 = clock();
+    Rprintf ("\t To go: ");
+    PrintfTime((double) (c1 - c0)/CLOCKS_PER_SEC/(iRun+1)*(iRuns-iRun-1));
+#endif
+    Rprintf ("\n");
   
     SEXP sexpTotTransitions;
     PROTECT(sexpTotTransitions = allocVector(INTSXP, 1));
@@ -522,7 +543,10 @@ SEXP GibsonBruck(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 
 #ifdef RB_TIME
   c1 = clock();
-  Rprintf ("Elapsed CPU time: %f\n", (float) (c1 - c0)/CLOCKS_PER_SEC);
+  double dCpuTime = (double) (c1 - c0)/CLOCKS_PER_SEC;
+  Rprintf ("Elapsed CPU time: ");
+  PrintfTime(dCpuTime);
+  Rprintf ("\t(%fs)\n", dCpuTime);
 #endif
 
   if (sexpRManage_time)
