@@ -1,19 +1,21 @@
-/* Copyright (C) 2009  Roberto Bertolusso, Marek Kimmel
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, a copy is available at
-   http://www.r-project.org/Licenses/
-*/
+// -*- mode: C; c-indent-level: 2; c-basic-offset: 2; tab-width: 8 -*-
+//
+// Copyright (C) 2009-2014 Roberto Bertolusso and Marek Kimmel
+//
+// This file is part of bioPN.
+//
+// bioPN is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// bioPN is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with bioPN. If not, see <http://www.gnu.org/licenses/>.
 
 #include <R.h>
 #include <Rinternals.h>
@@ -21,12 +23,13 @@
 #include <R_ext/Rdynload.h>
 #include <R_ext/Visibility.h>
 
-#include "quicksort.h"
+#include "helper.h"
 
 #define PRODUCTS_MAXIMUM_AMOUNT 100000000
 #define HAZARD_VALUE_TO_IGNORE 1e-15
 
 #define RB_TIME
+#define RB_SUBTIME
 
 #ifdef RB_TIME
 #include <time.h>
@@ -213,7 +216,11 @@ SEXP GillespieDirectCR(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
   SEXP sexpD_time;
   PROTECT(sexpD_time = allocVector(REALSXP, iTotalSteps));
   double *pdDiscTime = REAL(sexpD_time);
-  pdDiscTime[0] = 0;
+  double dTmp = 0;
+  for (k = 0; k < iTotalSteps; k++) {
+    pdDiscTime[k] = dTmp;
+    dTmp += dDelta;
+  }
 
   SEXP sexpMarkingRowNames;
   PROTECT(sexpMarkingRowNames = allocVector(INTSXP, iTotalSteps));
@@ -343,15 +350,8 @@ SEXP GillespieDirectCR(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
     int iTotTransitions = 0;
 
     int iStep = 0;
+    int iInterruptCnt = 10000000;
     do {
-      
-      if (iStep) {
-	--iStep;
-	for(iPlace = 0; iPlace < iPlaces; iPlace++) {
-	  pdCrntMarking[iPlace] = ppdMarking[iPlace][iStep];
-	}
-      }
-      
       if (pCManage_time || sexpRManage_time) {
 	double dEnd = 0;
 	if (pCManage_time) {
@@ -371,8 +371,13 @@ SEXP GillespieDirectCR(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 	}
 	iSectionSteps = (int)(dEnd / dDelta) + 1;
       }
-      
+
+      for(iPlace = 0; iPlace < iPlaces; iPlace++) {
+	ppdMarking[iPlace][iStep] = pdCrntMarking[iPlace];
+      }
+
       dTime = dTarget;
+      dTarget += dDelta;
       
       // For the calculation of all hazards...
       int iLastTransition = iTransitions;
@@ -391,8 +396,8 @@ SEXP GillespieDirectCR(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 		dNewHazard *= (pdCrntMarking[iPlace] - k) / (double)(k+1);
 	    }
 	    break;
-	  case HZ_CFUNCTION:	
-	    dNewHazard = ((double(*)(double *)) pCFunction[iTransition])(pdCrntMarking);
+	  case HZ_CFUNCTION:
+	    dNewHazard = ((double(*)(double, double *)) pCFunction[iTransition])(dTime, pdCrntMarking);
 	    break;
 	  case HZ_RFUNCTION:
 	    defineVar(install("y"), sexpCrntMarking, rho);
@@ -481,18 +486,23 @@ SEXP GillespieDirectCR(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 	iUsedRandomNumbers++;
 	
 	while (dTime >= dTarget) {
-	  if (!iRun)
-	    pdDiscTime[iStep] = dTarget;
+	  ++iStep;
 	  // Update the state for the fixed incremented time.
 	  for(iPlace = 0; iPlace < iPlaces; iPlace++)
 	    ppdMarking[iPlace][iStep] = pdCrntMarking[iPlace];
-	  if (++iStep >= iSectionSteps)
+	  if (iStep == iSectionSteps - 1)
 	    goto EXIT_LOOP;
+
 	  dTarget += dDelta;
+
+	  // Force check if user interrupted
+	  iInterruptCnt = 1;
+	}
+	if (! --iInterruptCnt) {
 	  // Allow user interruption
 	  R_CheckUserInterrupt();
+	  iInterruptCnt = 10000000;
 	}
-
 	do {
 	  // Find group containing firing transition
 	  double dRnd = unif_rand() * pnodeRoot->dPartialAcumHazard;
@@ -516,7 +526,11 @@ SEXP GillespieDirectCR(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 	double dMaxInGroup = dMinHazard * pow(2, iGroup + 1);
 	// Find transition in group
 	while (1) {
-	  //	  R_CheckUserInterrupt();
+	  if (! --iInterruptCnt) {
+	    // Allow user interruption
+	    R_CheckUserInterrupt();
+	    iInterruptCnt = 10000000;
+	  }
 	  iTransitionPtr = (int) (unif_rand() * piGroupElm[iGroup]);
 	  iUsedRandomNumbers++;
 	  iTransition = ppiGroup[iGroup][iTransitionPtr];
@@ -532,19 +546,20 @@ SEXP GillespieDirectCR(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 	    break;
 	  }
 	}
-      } while (++iTotTransitions);
+	++iTotTransitions;
+      } while (TRUE);
     EXIT_LOOP:;
       Rprintf(".");
     } while (iSectionSteps < iTotalSteps);
     iTotalUsedRandomNumbers += iUsedRandomNumbers;
-    Rprintf("\t%d\t%d\t%d\n", iTotTransitions, iUsedRandomNumbers, iTotalUsedRandomNumbers);
-  
-    /*
-    for(iTransition = 0; iTransition < iTransitions; iTransition++) {
-      Rprintf("%d\t%d\n", piOrderedTransition[iTransition], piTotTransitions[piOrderedTransition[iTransition]]);
-    }
-    */
-
+    Rprintf("\t%d\t%d\t%d", iTotTransitions, iUsedRandomNumbers, iTotalUsedRandomNumbers);
+#ifdef RB_SUBTIME
+    c1 = clock();
+    Rprintf ("\t To go: ");
+    PrintfTime((double) (c1 - c0)/CLOCKS_PER_SEC/(iRun+1)*(iRuns-iRun-1));
+#endif
+    Rprintf ("\n");
+    
     SEXP sexpTotTransitions;
     PROTECT(sexpTotTransitions = allocVector(INTSXP, 1));
     INTEGER(sexpTotTransitions)[0] = iTotTransitions;
@@ -592,7 +607,10 @@ SEXP GillespieDirectCR(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 
 #ifdef RB_TIME
   c1 = clock();
-  Rprintf ("Elapsed CPU time: %f\n", (float) (c1 - c0)/CLOCKS_PER_SEC);
+  double dCpuTime = (double) (c1 - c0)/CLOCKS_PER_SEC;
+  Rprintf ("Elapsed CPU time: ");
+  PrintfTime(dCpuTime);
+  Rprintf ("\t(%fs)\n", dCpuTime);
 #endif
 
   if (sexpRManage_time)
