@@ -1,19 +1,21 @@
-/* Copyright (C) 2010  Roberto Bertolusso, Marek Kimmel
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, a copy is available at
-   http://www.r-project.org/Licenses/
-*/
+// -*- mode: C; c-indent-level: 2; c-basic-offset: 2; tab-width: 8 -*-
+//
+// Copyright (C) 2009-2014 Roberto Bertolusso and Marek Kimmel
+//
+// This file is part of bioPN.
+//
+// bioPN is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// bioPN is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with bioPN. If not, see <http://www.gnu.org/licenses/>.
 
 #include <R.h>
 #include <Rinternals.h>
@@ -22,6 +24,7 @@
 #include <R_ext/Visibility.h>
 
 #include "quicksort.h"
+#include "helper.h"
 
 //#define RB_PRINT_INCR_INFO
 //#define RB_SAVE_INCR_INFO
@@ -76,6 +79,7 @@
 #define INCR_TO_SAVE 200000
 
 #define RB_TIME
+#define RB_SUBTIME
 
 #ifdef RB_TIME
 #include <time.h>
@@ -91,22 +95,10 @@ typedef enum {
 
 
 SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, SEXP delta,
-       SEXP runs, SEXP place, SEXP transition, SEXP ect, SEXP burnRnd, SEXP rho)
+		       SEXP runs, SEXP place, SEXP transition, SEXP ect, SEXP rho)
 {
   int k;
 
-  int iBurnRnd = *INTEGER(burnRnd);
-
-  if (iBurnRnd) {
-    Rprintf("\nBurning %d random numbers...", iBurnRnd);
-    GetRNGstate();
-    for (k = 0; k < iBurnRnd; k++) {
-      unif_rand();
-    }
-    PutRNGstate();
-    Rprintf(" done!\n", iBurnRnd);
-  }
-
 #ifdef RB_TIME
   clock_t c0, c1;
   c0 = clock();
@@ -315,7 +307,7 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
     error("Unrecognized time function type\n");
   }
   iTotalSteps = iSectionSteps = (int)(dT / dDelta) + 1;
-  
+
   // Hazard vector
   double *pdHazard = (double *) R_alloc(iTransitions, sizeof(double));
   double *pdBakHazard = (double *) R_alloc(iTransitions, sizeof(double));
@@ -331,7 +323,11 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
   SEXP sexpD_time;
   PROTECT(sexpD_time = allocVector(REALSXP, iTotalSteps));
   double *pdDiscTime = REAL(sexpD_time);
-  pdDiscTime[0] = 0;
+  double dTmp = 0;
+  for (k = 0; k < iTotalSteps; k++) {
+    pdDiscTime[k] = dTmp;
+    dTmp += dDelta;
+  }
   
   SEXP sexpMarkingRowNames;
   PROTECT(sexpMarkingRowNames = allocVector(INTSXP, iTotalSteps));
@@ -340,6 +336,7 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
     piTmp[k] = k+1;
   
   double **ppdMarking = (double **) R_alloc(iPlaces, sizeof(double *));
+  double **ppdHazard = (double **) R_alloc(iTransitions, sizeof(double *));
   
 #ifdef RB_SAVE_INCR_INFO
   double *pdIncr = (double *) R_alloc(INCR_TO_SAVE, sizeof(double));
@@ -367,8 +364,13 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
     SEXP sexpTotXTransition;
     PROTECT(sexpTotXTransition = allocVector(INTSXP, iTransitions));
     int *piTotTransitions = INTEGER(sexpTotXTransition);
+    SEXP sexpHazard;
+    PROTECT(sexpHazard = allocVector(VECSXP, iTransitions));
     
     for(iTransition = 0; iTransition < iTransitions; iTransition++) {
+      SET_VECTOR_ELT(sexpHazard, iTransition, sexpTmp = allocVector(REALSXP, iTotalSteps));
+      ppdHazard[iTransition] = REAL(sexpTmp);
+
       piTotTransitions[iTransition] = 0;
     }
     for(iTransitionPtr = 0; iTransitionPtr < iSlowTransitions; iTransitionPtr++) {
@@ -398,16 +400,9 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
     double dAbsoluteMaxError = 0;
 
     int iStep = 0;
-    int iAcceptedIncr = 0, iRejectedIncr = 0;
+    int iAcceptedIncr = 0, iRejectedIncr = 0, iInterruptCnt = 100000;
     double dNewHazard = 0;
     do {
-      if (iStep) {
-	--iStep;
-	for(iPlace = 0; iPlace < iPlaces; iPlace++) {
-	  pdCrntMarking[iPlace] = ppdMarking[iPlace][iStep];
-	}
-      }
-      
       if (pCManage_time || sexpRManage_time) {
 	double dEnd = 0;
 	if (pCManage_time) {
@@ -429,11 +424,12 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
       }
       
       for(iPlace = 0; iPlace < iPlaces; iPlace++) {
-	pdBakCrntMarking[iPlace] = pdTmpCrntMarking[iPlace] = pdCrntMarking[iPlace];
+	ppdMarking[iPlace][iStep] = pdBakCrntMarking[iPlace] = pdTmpCrntMarking[iPlace] = pdCrntMarking[iPlace];
       }
       
       dTime = dTarget;
-      
+      dTarget += dDelta;
+
       dStartIncr = dIncr;
       
       // For the calculation of all hazards...
@@ -441,7 +437,7 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
       
       double dAcumHazard = 0;
       for(iTransition = 0; iTransition < iTransitions; iTransition++) {
-	pdHazard[iTransition] = 0;
+	ppdHazard[iTransition][iStep] = pdHazard[iTransition] = 0;
       }
       
       do {    
@@ -494,6 +490,9 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
 	int iContinue = TRUE;
 	int iStartRK = 1;
 	do {
+	  if (dTime + dIncr > dTarget) {
+	    dIncr = dTarget - dTime;
+	  }
 	  for(iRKs = iStartRK; iRKs < 8; iRKs++) {
 	    switch (iRKs) {
 	    case 1:
@@ -589,7 +588,8 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
 		dNewHazard = REAL(eval(VECTOR_ELT(sexpFunction, iTransition), rho))[0];
 		break;
 	      }
-	      
+	      pdHazard[iTransition] = dNewHazard;
+
 	      for(iPlacePtr = 0; iPlacePtr < piSNZxRowTot[iTransition]; iPlacePtr++) {
 		iPlace = piSNZxRow[iTransition + iTransitions * iPlacePtr];
 		pdYdot[iPlace] += piS[iTransition + iTransitions * iPlace] * dNewHazard;
@@ -720,27 +720,35 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
 	    iStartRK = 1;
 	    continue;
 	  }
+
 	  // Update clock according to the last accepted increment
 	  dTime += dIncr;
-	  // Set new increment for next integration step
-	  dIncr = .8 * dIncr * R_pow(dTau/dInfNormError,err_pow);
 
-	  while (dTime >= dTarget) {
-	    if (!iRun)
-	      pdDiscTime[iStep] = dTarget;
+	  // Check if current state needs to be saved
+	  if (dTime == dTarget) {
+	    ++iStep;
 	    // Update the state for the fixed incremented time.
 	    for(iPlace = 0; iPlace < iPlaces; iPlace++) {
-	      ppdMarking[iPlace][iStep] = pdBakCrntMarking[iPlace];
+	      ppdMarking[iPlace][iStep] = pdCrntMarking[iPlace];
 	    }
-	    if (++iStep >= iSectionSteps)
+	    for(iTransition = 0; iTransition < iTransitions; iTransition++) {
+	      ppdHazard[iTransition][iStep] = pdHazard[iTransition];
+	    }
+	    if (iStep == iSectionSteps - 1)
 	      goto EXIT_LOOP;
 	    
 	    dTarget += dDelta;
 	    
+	    // Force check if user interrupted
+	    iInterruptCnt = 1;
+	  }
+	  if (! --iInterruptCnt) {
 	    // Allow user interruption
 	    R_CheckUserInterrupt();
+	    iInterruptCnt = 100000;
 	  }
-	  
+	  // Set new increment for next integration step
+	  dIncr = .8 * dIncr * R_pow(dTau/dInfNormError,err_pow);
 	} while (iContinue);
 #ifdef RB_PRINT_INCR_INFO
 	Rprintf("Find Transition\n", dIncr);
@@ -779,7 +787,8 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
 	    break;
 	  }
 	}
-      } while (++iTotTransitions);
+	++iTotTransitions;
+      } while (TRUE);
     EXIT_LOOP:;
       Rprintf(".");
     } while (iSectionSteps < iTotalSteps);
@@ -792,6 +801,11 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
     Rprintf("\t%d\t%d\t%d\t%e\t%e", iTotRejIncr, iTotGoBackIncr, iTotAccpIncr, dMeanAccpIncr, dSdAccpIncr);
 #endif
 
+#ifdef RB_SUBTIME
+    c1 = clock();
+    Rprintf ("\t To go: ");
+    PrintfTime((double) (c1 - c0)/CLOCKS_PER_SEC/(iRun+1)*(iRuns-iRun-1));
+#endif
     Rprintf("\n");
 
     SEXP sexpTotTransitions;
@@ -805,20 +819,22 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
     SEXP sexpThisRun;
 #ifdef RB_SAVE_INCR_INFO
     if (iRun >= 10)
-      PROTECT(sexpThisRun = allocVector(VECSXP, 4));
+      PROTECT(sexpThisRun = allocVector(VECSXP, 5));
     else
-      PROTECT(sexpThisRun = allocVector(VECSXP, 9));
+      PROTECT(sexpThisRun = allocVector(VECSXP, 10));
 #else
-    PROTECT(sexpThisRun = allocVector(VECSXP, 4));
+    PROTECT(sexpThisRun = allocVector(VECSXP, 5));
 #endif
     
     SET_VECTOR_ELT(sexpThisRun, 0, sexpMarking);
     UNPROTECT_PTR(sexpMarking);
-    SET_VECTOR_ELT(sexpThisRun, 1, sexpTotXTransition);
+    SET_VECTOR_ELT(sexpThisRun, 1, sexpHazard);
+    UNPROTECT_PTR(sexpHazard);
+    SET_VECTOR_ELT(sexpThisRun, 2, sexpTotXTransition);
     UNPROTECT_PTR(sexpTotXTransition);
-    SET_VECTOR_ELT(sexpThisRun, 2, sexpTotTransitions);
+    SET_VECTOR_ELT(sexpThisRun, 3, sexpTotTransitions);
     UNPROTECT_PTR(sexpTotTransitions);
-    SET_VECTOR_ELT(sexpThisRun, 3, sexpUsedRandomNumbers);
+    SET_VECTOR_ELT(sexpThisRun, 4, sexpUsedRandomNumbers);
     UNPROTECT_PTR(sexpUsedRandomNumbers);
 #ifdef RB_SAVE_INCR_INFO
     if (iRun < 10) {
@@ -829,59 +845,60 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
       int i;
       for (i = 0; i < iTotIncrTime; i++)
 	pdTmp[i] = pdIncr[i];
-      SET_VECTOR_ELT(sexpThisRun, 4, sexpTmp);
+      SET_VECTOR_ELT(sexpThisRun, 5, sexpTmp);
       UNPROTECT_PTR(sexpTmp);
       
       PROTECT(sexpTmp = allocVector(REALSXP, iTotIncrTime));
       pdTmp = REAL(sexpTmp);
       for (i = 0; i < iTotIncrTime; i++)
 	pdTmp[i] = pdIncrTime[i];
-      SET_VECTOR_ELT(sexpThisRun, 5, sexpTmp);
-      UNPROTECT_PTR(sexpTmp);
-      
-      PROTECT(sexpTmp = allocVector(REALSXP, iTotIntHazardTime));
-      pdTmp = REAL(sexpTmp);
-      for (i = 0; i < iTotIntHazardTime; i++)
-	pdTmp[i] = pdAcumHazard[i];
       SET_VECTOR_ELT(sexpThisRun, 6, sexpTmp);
       UNPROTECT_PTR(sexpTmp);
       
       PROTECT(sexpTmp = allocVector(REALSXP, iTotIntHazardTime));
       pdTmp = REAL(sexpTmp);
       for (i = 0; i < iTotIntHazardTime; i++)
-	pdTmp[i] = pdIntHazard[i];
+	pdTmp[i] = pdAcumHazard[i];
       SET_VECTOR_ELT(sexpThisRun, 7, sexpTmp);
       UNPROTECT_PTR(sexpTmp);
       
       PROTECT(sexpTmp = allocVector(REALSXP, iTotIntHazardTime));
       pdTmp = REAL(sexpTmp);
       for (i = 0; i < iTotIntHazardTime; i++)
-	pdTmp[i] = pdIntHazardTime[i];
+	pdTmp[i] = pdIntHazard[i];
       SET_VECTOR_ELT(sexpThisRun, 8, sexpTmp);
       UNPROTECT_PTR(sexpTmp);
+      
+      PROTECT(sexpTmp = allocVector(REALSXP, iTotIntHazardTime));
+      pdTmp = REAL(sexpTmp);
+      for (i = 0; i < iTotIntHazardTime; i++)
+	pdTmp[i] = pdIntHazardTime[i];
+      SET_VECTOR_ELT(sexpThisRun, 9, sexpTmp);
+      UNPROTECT_PTR(sexpTmp);
     }
 #endif
     
     SEXP sexpNames;
 #ifdef RB_SAVE_INCR_INFO
     if (iRun >= 10)
-      PROTECT(sexpNames = allocVector(VECSXP, 4));
+      PROTECT(sexpNames = allocVector(VECSXP, 5));
     else
-      PROTECT(sexpNames = allocVector(VECSXP, 9));
+      PROTECT(sexpNames = allocVector(VECSXP, 10));
 #else
-    PROTECT(sexpNames = allocVector(VECSXP, 4));
+    PROTECT(sexpNames = allocVector(VECSXP, 5));
 #endif
     SET_VECTOR_ELT(sexpNames, 0, mkChar("M"));
-    SET_VECTOR_ELT(sexpNames, 1, mkChar("transitions"));
-    SET_VECTOR_ELT(sexpNames, 2, mkChar("tot.transitions"));
-    SET_VECTOR_ELT(sexpNames, 3, mkChar("tot.rnd"));
+    SET_VECTOR_ELT(sexpNames, 1, mkChar("hazard"));
+    SET_VECTOR_ELT(sexpNames, 2, mkChar("transitions"));
+    SET_VECTOR_ELT(sexpNames, 3, mkChar("tot.transitions"));
+    SET_VECTOR_ELT(sexpNames, 4, mkChar("tot.rnd"));
 #ifdef RB_SAVE_INCR_INFO
     if (iRun < 10) {
-      SET_VECTOR_ELT(sexpNames, 4, mkChar("incr"));
-      SET_VECTOR_ELT(sexpNames, 5, mkChar("incr.time"));
-      SET_VECTOR_ELT(sexpNames, 6, mkChar("hazard"));
-      SET_VECTOR_ELT(sexpNames, 7, mkChar("int.hazard"));
-      SET_VECTOR_ELT(sexpNames, 8, mkChar("int.hazard.time"));
+      SET_VECTOR_ELT(sexpNames, 5, mkChar("incr"));
+      SET_VECTOR_ELT(sexpNames, 6, mkChar("incr.time"));
+      SET_VECTOR_ELT(sexpNames, 7, mkChar("tot.hazard"));
+      SET_VECTOR_ELT(sexpNames, 8, mkChar("int.tot.hazard"));
+      SET_VECTOR_ELT(sexpNames, 9, mkChar("int.tot.hazard.time"));
     }
 #endif
     setAttrib(sexpThisRun, R_NamesSymbol, sexpNames);
@@ -912,7 +929,10 @@ SEXP HaseltineRawlings(SEXP pre, SEXP post, SEXP h, SEXP slow, SEXP M, SEXP T, S
 
 #ifdef RB_TIME
   c1 = clock();
-  Rprintf ("Elapsed CPU time: %f\n", (float) (c1 - c0)/CLOCKS_PER_SEC);
+  double dCpuTime = (double) (c1 - c0)/CLOCKS_PER_SEC;
+  Rprintf ("Elapsed CPU time: ");
+  PrintfTime(dCpuTime);
+  Rprintf ("\t(%fs)\n", dCpuTime);
 #endif
 
   if (sexpRManage_time)
