@@ -1,19 +1,21 @@
-/* Copyright (C) 2009  Roberto Bertolusso, Marek Kimmel
-
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 3 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with this library; if not, a copy is available at
-   http://www.r-project.org/Licenses/
-*/
+// -*- mode: C; c-indent-level: 2; c-basic-offset: 2; tab-width: 8 -*-
+//
+// Copyright (C) 2009-2014 Roberto Bertolusso and Marek Kimmel
+//
+// This file is part of bioPN.
+//
+// bioPN is free software: you can redistribute it and/or modify it
+// under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 2 of the License, or
+// (at your option) any later version.
+//
+// bioPN is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with bioPN. If not, see <http://www.gnu.org/licenses/>.
 
 #include <R.h>
 #include <Rinternals.h>
@@ -22,6 +24,7 @@
 #include <R_ext/Visibility.h>
 
 #include "quicksort.h"
+#include "helper.h"
 
 //#define RB_PRINT_INCR_INFO
 #define RB_SAVE_INCR_INFO
@@ -30,6 +33,7 @@
 #define ZERO 1e-20
 
 #define RB_TIME
+#define RB_SUBTIME
 
 #ifdef RB_TIME
 #include <time.h>
@@ -45,23 +49,11 @@ typedef enum {
 
 
 SEXP PartitionedLeaping(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
-			SEXP runs, SEXP place, SEXP transition, SEXP ect, SEXP burnRnd, SEXP rho)
+			SEXP runs, SEXP place, SEXP transition, SEXP ect, SEXP rho)
 {
   int k;
   double dTmp;
 
-  int iBurnRnd = *INTEGER(burnRnd);
-
-  if (iBurnRnd) {
-    Rprintf("\nBurning %d random numbers...", iBurnRnd);
-    GetRNGstate();
-    for (k = 0; k < iBurnRnd; k++) {
-      unif_rand();
-    }
-    PutRNGstate();
-    Rprintf(" done!\n", iBurnRnd);
-  }
-
 #ifdef RB_TIME
   clock_t c0, c1;
   c0 = clock();
@@ -365,7 +357,11 @@ SEXP PartitionedLeaping(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
   SEXP sexpD_time;
   PROTECT(sexpD_time = allocVector(REALSXP, iTotalSteps));
   double *pdDiscTime = REAL(sexpD_time);
-  pdDiscTime[0] = 0;
+  dTmp = 0;
+  for (k = 0; k < iTotalSteps; k++) {
+    pdDiscTime[k] = dTmp;
+    dTmp += dDelta;
+  }
 
   SEXP sexpMarkingRowNames;
   PROTECT(sexpMarkingRowNames = allocVector(INTSXP, iTotalSteps));
@@ -393,9 +389,7 @@ SEXP PartitionedLeaping(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
   for (iRun = 0; iRun < iRuns; iRun++) {
 
 #ifdef RB_SAVE_INCR_INFO
-    int iTotAccpIncr = 0, iTotRejIncr = 0, iTotGoBackIncr = 0, iTotIntHazardTime = 0, iTotIncrTime = 0;
-    double dSumAccpIncr = 0;
-    double dSumSqAccpIncr = 0;
+    int iTotIntHazardTime = 0, iTotIncrTime = 0;
 #endif
  
     int iUsedRandomNumbers = 0;
@@ -431,10 +425,8 @@ SEXP PartitionedLeaping(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
     double dTime, dTarget = 0;
     int iTotTransitions = 0;
     
-    double dIncr = MIN_INCR, dStartIncr;
-
-    double dAbsoluteMaxDiff = 0;
     int iStep = 0;
+    int iInterruptCnt = 10000000;
     double dNewHazard = 0;
     do {
       if (iStep) {
@@ -470,8 +462,6 @@ SEXP PartitionedLeaping(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 
       dTime = dTarget;
 
-      dStartIncr = dIncr;
-
       for(iTransition = 0; iTransition < iTransitions; iTransition++) {
 	pdHazard[iTransition] = 0;
       }
@@ -663,8 +653,6 @@ SEXP PartitionedLeaping(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 	}
 	
 	while (dTime >= dTarget) {
-	  if (!iRun)
-	    pdDiscTime[iStep] = dTarget;
 	  // Update the state for the fixed incremented time.
 	  for(iPlace = 0; iPlace < iPlaces; iPlace++) {
 	    ppdMarking[iPlace][iStep] = pdBakCrntMarking[iPlace];
@@ -673,25 +661,28 @@ SEXP PartitionedLeaping(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 	    goto EXIT_LOOP;
 	  
 	  dTarget += dDelta;
-	  
+
+	  // Force check if user interrupted
+	  iInterruptCnt = 1;
+	}
+	if (! --iInterruptCnt) {
 	  // Allow user interruption
 	  R_CheckUserInterrupt();
+	  iInterruptCnt = 10000000;
 	}
       } while (++iTotTransitions);
     EXIT_LOOP:;
       Rprintf(".");
     } while (iSectionSteps < iTotalSteps);
     iTotalUsedRandomNumbers += iUsedRandomNumbers;
-    Rprintf("\t%d\t%d\t%e", iUsedRandomNumbers, iTotalUsedRandomNumbers, dAbsoluteMaxDiff);
-
-#ifdef RB_SAVE_INCR_INFO
-    double dMeanAccpIncr = dSumAccpIncr/iTotAccpIncr;
-    double dSdAccpIncr = sqrt(dSumSqAccpIncr/iTotAccpIncr - dMeanAccpIncr*dMeanAccpIncr); 
-    Rprintf("\t%d\t%d\t%d\t%e\t%e", iTotRejIncr, iTotGoBackIncr, iTotAccpIncr, dMeanAccpIncr, dSdAccpIncr);
+    Rprintf("\t%d\t%d\t%d", iTotTransitions, iUsedRandomNumbers, iTotalUsedRandomNumbers);
+#ifdef RB_SUBTIME
+    c1 = clock();
+    Rprintf ("\t To go: ");
+    PrintfTime((double) (c1 - c0)/CLOCKS_PER_SEC/(iRun+1)*(iRuns-iRun-1));
 #endif
-
-    Rprintf("\n");
-
+    Rprintf ("\n");
+  
     SEXP sexpTotTransitions;
     PROTECT(sexpTotTransitions = allocVector(INTSXP, 1));
     INTEGER(sexpTotTransitions)[0] = iTotTransitions;
@@ -810,7 +801,10 @@ SEXP PartitionedLeaping(SEXP pre, SEXP post, SEXP h, SEXP M, SEXP T, SEXP delta,
 
 #ifdef RB_TIME
   c1 = clock();
-  Rprintf ("Elapsed CPU time: %f\n", (float) (c1 - c0)/CLOCKS_PER_SEC);
+  double dCpuTime = (double) (c1 - c0)/CLOCKS_PER_SEC;
+  Rprintf ("Elapsed CPU time: ");
+  PrintfTime(dCpuTime);
+  Rprintf ("\t(%fs)\n", dCpuTime);
 #endif
 
   if (sexpRManage_time)
